unit CommonfmeOptions_PasswordExpiry;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  OTFEFreeOTFEBase_U, CommonSettings, SDUFrames, SDUFilenameEdit_U,
  OTFEFreeOTFE_VolumeSelect, CommonfmeOptions_Base, Spin64;

type
  TfmeOptions_PasswordExpiry = class(TfmeOptions_Base)
    gbPasswordExpiry: TGroupBox;
    ckPassExpiryExpire: TCheckBox;
    ckPassExpiryWarning: TCheckBox;
    sePassExpiryExpire_Days: TSpinEdit64;
    sePassExpiryWarning_Days: TSpinEdit64;
    lblPassExpiryExpire_Days_Post: TLabel;
    lblPassExpiryWarning_Days_Post: TLabel;
    lblPassExpiryExpire_Days_Pre: TLabel;
    lblPassExpiryWarning_Days_Pre: TLabel;
    procedure ControlChanged(Sender: TObject);
  private

  public
    OTFEFreeOTFE: TOTFEFreeOTFEBase;

    procedure EnableDisableControls(); override;
    procedure Initialize(); override;
    procedure ReadSettings(config: TSettings); override;
    procedure WriteSettings(config: TSettings); override;
    function  CheckSettings(): boolean; override;
  end;

implementation

{$R *.dfm}

uses
  SDUi18n,
  SDUGeneral,
  SDUDialogs,
  OTFEFreeOTFEDLL_U;

{$IFDEF _NEVER_DEFINED}
// This is just a dummy const to fool dxGetText when extracting message
// information
// This const is never used; it's #ifdef'd out - SDUCRLF in the code refers to
// picks up SDUGeneral.SDUCRLF
const
  SDUCRLF = ''#13#10;
{$ENDIF}
  
resourcestring
  FILTER_LIBRARY_FILES = 'Library files (*.dll)|*.dll|All files|*.*';

procedure TfmeOptions_PasswordExpiry.ControlChanged(Sender: TObject);
begin
  inherited;
  EnableDisableControls();

end;

procedure TfmeOptions_PasswordExpiry.Initialize();
begin
  SDUCenterControl(gbPasswordExpiry, ccHorizontal);
  SDUCenterControl(gbPasswordExpiry, ccVertical, 25);

end;

procedure TfmeOptions_PasswordExpiry.ReadSettings(config: TSettings);
begin
  ckPassExpiryExpire.checked := config.OptPassExpiryExpire;
  sePassExpiryExpire_Days.value := config.OptPassExpiryExpire_Days;
  ckPassExpiryWarning.checked := config.OptPassExpiryWarning;
  sePassExpiryWarning_Days.value := config.OptPassExpiryWarning_Days;

end;

procedure TfmeOptions_PasswordExpiry.WriteSettings(config: TSettings);
begin
  config.OptPassExpiryExpire := ckPassExpiryExpire.checked;
  config.OptPassExpiryExpire_Days := sePassExpiryExpire_Days.value;
  config.OptPassExpiryWarning := ckPassExpiryWarning.checked;
  config.OptPassExpiryWarning_Days := sePassExpiryWarning_Days.value;

end;

function TfmeOptions_PasswordExpiry.CheckSettings(): boolean;
var
  allOK: boolean;
begin
  allOK := inherited CheckSettings();

  if allOK then
    begin
    if ckPassExpiryExpire.checked then
      begin
      if (sePassExpiryExpire_Days.Value < 1) then
        begin
        SDUMessageDlg(
                      _('If password expiry is enabled, the number of days before passwords expire must be set'),
                      mtError
                     );
        allOK := FALSE;
        end;
      end;
    end;

  if allOK then
    begin
    if ckPassExpiryWarning.checked then
      begin
      if (sePassExpiryWarning_Days.Value < 1) then
        begin
        SDUMessageDlg(
                      _('If password expiry warnings are enabled, the number of days before the warning is shown must be set.'),
                      mtError
                     );
        allOK := FALSE;
        end;
      end;
    end;

  if allOK then
    begin
    if (
        ckPassExpiryExpire.checked and
        ckPassExpiryWarning.checked
       ) then
      begin
      if (sePassExpiryWarning_Days.Value > sePassExpiryExpire_Days.Value) then
        begin
        SDUMessageDlg(
                      _('If both password expiry and warnings are enabled, the number of days before passwords expire must be greater than the number of days before warnings are shown.'),
                      mtError
                     );
        allOK := FALSE;
        end;
      end;
    end;

  Result := allOK;
end;

procedure TfmeOptions_PasswordExpiry.EnableDisableControls();
begin
  inherited;

  SDUEnableControl(sePassExpiryExpire_Days, ckPassExpiryExpire.checked);
  SDUEnableControl(lblPassExpiryExpire_Days_Pre, ckPassExpiryExpire.checked);
  SDUEnableControl(lblPassExpiryExpire_Days_Post, ckPassExpiryExpire.checked);
  SDUEnableControl(sePassExpiryWarning_Days, ckPassExpiryWarning.checked);
  SDUEnableControl(lblPassExpiryWarning_Days_Pre, ckPassExpiryWarning.checked);
  SDUEnableControl(lblPassExpiryWarning_Days_Post, ckPassExpiryWarning.checked);

end;

END.

